## @package haar
#  Provides functions for finding faces in the reference picture.

import cv2.cv as cv
import Image
import glob
import os
import time
import picamera
# suppress warnings
import warnings
warnings.filterwarnings('default', category=DeprecationWarning)

## Finds face in photo.
def DetectFace(image, faceCascade, returnImage=False):

    min_size = (20,20)
    haar_scale = 1.1
    min_neighbors = 3
    haar_flags = 0

    cv.EqualizeHist(image, image)

    faces = cv.HaarDetectObjects(
            image, faceCascade, cv.CreateMemStorage(0),
            haar_scale, min_neighbors, haar_flags, min_size
        )

    if faces and returnImage:
        for ((x, y, w, h), n) in faces:
            pt1 = (int(x), int(y))
            pt2 = (int(x + w), int(y + h))
            cv.Rectangle(image, pt1, pt2, cv.RGB(255, 0, 0), 5, 8, 0)

    if returnImage:
        return image
    else:
        return faces

## Converts photo to gray scale.
def pil2cvGrey(pil_im):
    pil_im = pil_im.convert('L')
    cv_im = cv.CreateImageHeader(pil_im.size, cv.IPL_DEPTH_8U, 1)
    cv.SetData(cv_im, pil_im.tostring(), pil_im.size[0]  )
    return cv_im

## Converts image to PIL.
def cv2pil(cv_im):
    return Image.fromstring("L", cv.GetSize(cv_im), cv_im.tostring())

## Crops image to the given bounding box.
def imgCrop(image, cropBox, boxScale=1):
    
    xDelta=max(cropBox[2]*(boxScale-1),0)
    yDelta=max(cropBox[3]*(boxScale-1),0)

    PIL_box=[cropBox[0]-xDelta, cropBox[1]-yDelta, cropBox[0]+cropBox[2]+xDelta, cropBox[1]+cropBox[3]+yDelta]

    return image.crop(PIL_box)

## Extracts the actual face and stores it.
#
#  Loads detection pattern according to the Haar Cascade principal.
#  Detects actual face and crops the image.
#  Stores the final image as pgm file.
def faceCrop(imagePattern,boxScale=1):
    faceCascade = cv.Load('haarcascade_frontalface_alt.xml')

    imgList=glob.glob(imagePattern)
    if len(imgList)<=0:
        print ('No Images Found')
        return

    for img in imgList:
        pil_im=Image.open(img)
        cv_im=pil2cvGrey(pil_im)
        faces=DetectFace(cv_im,faceCascade)
        if faces:
            n=1
            for face in faces:
                pil_im=cv2pil(cv_im)
                croppedImage=imgCrop(pil_im, face[0],boxScale=boxScale)
                fname,ext=os.path.splitext(img)
                cropped=croppedImage.resize((92, 112), Image.ANTIALIAS)
                cropped.save(fname+'.pgm')
                n+=1
        else:
            print ('No faces found:', img)
            
