## @package training
#  Provides functions for generation of the pictures.
#
#  Each time, 5 photos are being taken and stored in the database.
#  These photos are being replaced by cropped faces in gray scale color space 

import cv2.cv as cv
import glob
import os
import time
import picamera
from Tkinter import *
import Image
# suppress warnings
import warnings
warnings.filterwarnings('default', category=DeprecationWarning)

## Takes 5 photos in jpg format in intervals of 2 seconds.
#
#  Creates directory with corresponding name, if not present.
#  Stores photos in corresponding directory. 
def takePictures(name):
    if not os.path.exists('pictures/' + name):
        os.makedirs('pictures/' + name)
    with picamera.PiCamera() as camera:
        camera.resolution = (500, 400)
        camera.start_preview()
        for i in range(1, 6):
            time.sleep(2)
            camera.capture('pictures/' + name + '/' + str(i) +'.jpg')
        camera.stop_preview()

## Finds faces in the photos.
def DetectFace(image, faceCascade, returnImage=False):
    
    min_size = (20,20)
    haar_scale = 1.1
    min_neighbors = 3
    haar_flags = 0

    cv.EqualizeHist(image, image)

    faces = cv.HaarDetectObjects(
            image, faceCascade, cv.CreateMemStorage(0),
            haar_scale, min_neighbors, haar_flags, min_size
        )

    if faces and returnImage:
        for ((x, y, w, h), n) in faces:

            pt1 = (int(x), int(y))
            pt2 = (int(x + w), int(y + h))
            cv.Rectangle(image, pt1, pt2, cv.RGB(255, 0, 0), 5, 8, 0)

    if returnImage:
        return image
    else:
        return faces

## Convertes photo to gray scale.
def pil2cvGrey(pil_im):
    pil_im = pil_im.convert('L')
    cv_im = cv.CreateImageHeader(pil_im.size, cv.IPL_DEPTH_8U, 1)
    cv.SetData(cv_im, pil_im.tostring(), pil_im.size[0]  )
    return cv_im

## convertes image to PIL format.
def cv2pil(cv_im):
    return Image.fromstring("L", cv.GetSize(cv_im), cv_im.tostring())

## Crops image to the given bounding box.
def imgCrop(image, cropBox, boxScale=1):

    xDelta=max(cropBox[2]*(boxScale-1),0)
    yDelta=max(cropBox[3]*(boxScale-1),0)

    PIL_box=[cropBox[0]-xDelta, cropBox[1]-yDelta, cropBox[0]+cropBox[2]+xDelta, cropBox[1]+cropBox[3]+yDelta]

    return image.crop(PIL_box)

## Extracts the actual face and stores it.
#
#  Loads detection pattern according to the Haar Cascade principal.
#  Detects actual face and crops the image.
#  Stores the final image as pgm file.
def faceCrop(imagePattern,boxScale=1):
    
    faceCascade = cv.Load('haarcascade_frontalface_alt.xml')

    imgList=glob.glob(imagePattern)
    if len(imgList)<=0:
        print ('No Images Found')
        return

    for img in imgList:
        pil_im=Image.open(img)
        cv_im=pil2cvGrey(pil_im)
        faces=DetectFace(cv_im,faceCascade)
        if faces:
            n=1
            for face in faces:
                pil_im=cv2pil(cv_im)
                croppedImage=imgCrop(pil_im, face[0],boxScale=boxScale)
                fname,ext=os.path.splitext(img)
                cropped=croppedImage.resize((92, 112), Image.ANTIALIAS)
                cropped.save(fname+'.pgm')
                n+=1
        else:
            print ('No faces found:', img)
        
        os.remove(img)


        